# Generate devel rpm
%global with_devel 1
# Build project from bundled dependencies
%global with_bundled 0
# Build with debug info rpm
%global with_debug 1
# Run tests in check section
%global with_check 1
# Generate unit-test rpm
%global with_unit_test 1

%if 0%{?with_debug}
%global _dwz_low_mem_die_limit 0
%else
%global debug_package   %{nil}
%endif

%if ! 0%{?gobuild:1}
%global gobuild(o:) go build -ldflags "${LDFLAGS:-} -B 0x$(head -c20 /dev/urandom|od -An -tx1|tr -d ' \\n')" -a -v -x %{?**};
%endif

%global provider        github
%global provider_tld    com
%global project         syncthing
%global repo            syncthing-inotify
%global long_name       golang-%{provider}-%{project}-%{repo}
# https://github.com/syncthing/syncthing-inotify
%global provider_prefix %{provider}.%{provider_tld}/%{project}/%{repo}
%global import_path     %{provider_prefix}
%global commit          f628a84902f9ffdf9cb439294f83383b60d7efae
%global shortcommit     %(c=%{commit}; echo ${c:0:7})

# commit f628a84902f9ffdf9cb439294f83383b60d7efae == version 0.8.7


Name:           syncthing-inotify
Version:        0.8.7
Release:        4%{?dist}
Summary:        Syncthing File watcher
License:        MPLv2.0

URL:            https://github.com/%{project}/%{repo}
Source0:        %{url}/archive/v%{version}/%{name}-%{version}.tar.gz

# Use proposed patch to fix compatibility with syncthing 0.14.33+
# https://github.com/syncthing/syncthing-inotify/issues/175
# https://github.com/syncthing/syncthing-inotify/pull/176
Patch0:         00-syncthing-01433-bump.patch

# e.g. el6 has ppc64 arch without gcc-go, so EA tag is required
ExclusiveArch:  %{?go_arches:%{go_arches}}%{!?go_arches:%{ix86} x86_64 aarch64 %{arm}}
# If go_compiler is not set to 1, there is no virtual provide. Use golang instead.
BuildRequires:  %{?go_compiler:compiler(go-compiler)}%{!?go_compiler:golang}

BuildRequires:  systemd
%{?systemd_requires}

%if ! 0%{?with_bundled}
BuildRequires:  golang(github.com/cenkalti/backoff)
BuildRequires:  golang(github.com/zillode/notify)
BuildRequires:  golang(github.com/syncthing/syncthing/lib/ignore)
%endif

Provides:       %{long_name} = %{version}-%{release}

Requires:       syncthing >= 0.14.33


%description
Syncthing (core) uses a rescan interval to detect changes in folders.
This application (syncthing-inotify) uses OS primitives to detect
changes as soon as they happen. Therefore, if you save a file,
syncthing-inotify will know about it and pass this information to
Syncthing such that near real-time synchronisation can be achieved.


%if 0%{?with_devel}
%package        devel
Summary:        Syncthing File watcher (development files)
Provides:       %{long_name}-devel = %{version}-%{release}
BuildArch:      noarch

Provides:       golang(%{import_path}) = %{version}-%{release}

%description    devel
Syncthing (core) uses a rescan interval to detect changes in folders.
This application (syncthing-inotify) uses OS primitives to detect
changes as soon as they happen. Therefore, if you save a file,
syncthing-inotify will know about it and pass this information to
Syncthing such that near real-time synchronisation can be achieved.

This package contains the syncthing-inotify source files, which are
needed as dependency for building packages using syncthing-inotify.
%endif


%if 0%{?with_unit_test} && 0%{?with_devel}
%package        unit-test-devel
Summary:        Syncthing File watcher (unit tests)

# test subpackage tests code from devel subpackage
Requires:       %{name}-devel = %{version}-%{release}

%description    unit-test-devel
Syncthing (core) uses a rescan interval to detect changes in folders.
This application (syncthing-inotify) uses OS primitives to detect
changes as soon as they happen. Therefore, if you save a file,
syncthing-inotify will know about it and pass this information to
Syncthing such that near real-time synchronisation can be achieved.

This package contains the unit tests for syncthing-inotify.
%endif


%prep
%autosetup -p1


%build
# remove bundled libraries
rm -r vendor

# prepare build environment
mkdir -p ./_build/src/%{provider}.%{provider_tld}/%{project}
ln -s $(pwd) ./_build/src/%{import_path}

# run build and set version correctly
export GOPATH=$(pwd)/_build:%{gopath}
export LDFLAGS="-X main.Version=%{version}"
%gobuild -o %{name} %{import_path}


%install
# install binary
mkdir -p %{buildroot}/%{_bindir}
cp -pav %{name} %{buildroot}/%{_bindir}/%{name}

# install systemd units
mkdir -p %{buildroot}/%{_unitdir}
mkdir -p %{buildroot}/%{_userunitdir}

cp -pav etc/linux-systemd/user/%{name}.service %{buildroot}/%{_userunitdir}/
cp -pav etc/linux-systemd/system/%{name}@.service %{buildroot}/%{_unitdir}/

# install systemd preset disabling the service per default
mkdir -p %{buildroot}/%{_prefix}/lib/systemd/user-preset
echo "disable syncthing-inotify*" > %{buildroot}/%{_prefix}/lib/systemd/user-preset/90-syncthing-inotify.preset

# source codes for building projects
%if 0%{?with_devel}
install -d -p %{buildroot}/%{gopath}/src/%{import_path}/
echo "%%dir %%{gopath}/src/%%{import_path}/." >> devel.file-list

# find all *.go but no *_test.go files and generate devel.file-list
for file in $(find . \( -iname "*.go" -or -iname "*.s" \) \! -iname "*_test.go") ; do
    dirprefix=$(dirname $file)
    install -d -p %{buildroot}/%{gopath}/src/%{import_path}/$dirprefix
    cp -pav $file %{buildroot}/%{gopath}/src/%{import_path}/$file
    echo "%%{gopath}/src/%%{import_path}/$file" >> devel.file-list

    while [ "$dirprefix" != "." ]; do
        echo "%%dir %%{gopath}/src/%%{import_path}/$dirprefix" >> devel.file-list
        dirprefix=$(dirname $dirprefix)
    done
done
%endif

# testing files for this project
%if 0%{?with_unit_test} && 0%{?with_devel}
install -d -p %{buildroot}/%{gopath}/src/%{import_path}/

# find all *_test.go files and generate unit-test-devel.file-list
for file in $(find . -iname "*_test.go") ; do
    dirprefix=$(dirname $file)
    install -d -p %{buildroot}/%{gopath}/src/%{import_path}/$dirprefix
    cp -pav $file %{buildroot}/%{gopath}/src/%{import_path}/$file
    echo "%%{gopath}/src/%%{import_path}/$file" >> unit-test-devel.file-list

    while [ "$dirprefix" != "." ]; do
        echo "%%dir %%{gopath}/src/%%{import_path}/$dirprefix" >> devel.file-list
        dirprefix=$(dirname $dirprefix)
    done
done
%endif

%if 0%{?with_devel}
sort -u -o devel.file-list devel.file-list
%endif


%check
%if 0%{?with_check} && 0%{?with_unit_test} && 0%{?with_devel}
export GOPATH=%{buildroot}/%{gopath}:%{gopath}

%if ! 0%{?gotest:1}
%global gotest go test
%endif

# Some tests fail randomly, this has been reported upstream at:
# https://github.com/syncthing/syncthing-inotify/issues/150
# %%gotest %%{import_path}
%endif


%post
%systemd_post 'syncthing-inotify@*.service'
%systemd_user_post syncthing-inotify.service

%preun
%systemd_preun 'syncthing-inotify@*.service'
%systemd_user_preun syncthing-inotify.service

%postun
%systemd_postun_with_restart 'syncthing-inotify@*.service'
%systemd_user_postun_with_restart syncthing-inotify.service


#define license tag if not already defined
%{!?_licensedir:%global license %doc}


%files
%license LICENSE
%doc README.md

%{_bindir}/%{name}

%{_unitdir}/%{name}@.service
%{_userunitdir}/%{name}.service

%{_prefix}/lib/systemd/user-preset/90-%{name}.preset


%if 0%{?with_devel}
%files devel -f devel.file-list
%license LICENSE
%doc README.md
%dir %{gopath}/src/%{provider}.%{provider_tld}/%{project}
%endif


%if 0%{?with_unit_test} && 0%{?with_devel}
%files unit-test-devel -f unit-test-devel.file-list
%license LICENSE
%doc README.md
%endif


%changelog
* Fri Aug 18 2017 Fabio Valentini <decathorpe@gmail.com> - 0.8.7-4
- Add links to compatibility bug report and pull request.
- Fix wrong access permissions on patch.
- Remove Supplements: syncthing.

* Thu Aug 17 2017 Fabio Valentini <decathorpe@gmail.com> - 0.8.7-3
- Skip tests for now, since they are failing randomly.
- Build with correct ldflags to fix debug packages.
- Add user-preset disabling the service by default, just to be safe.

* Fri Aug 11 2017 Fabio Valentini <decathorpe@gmail.com> - 0.8.7-2
- Add patch to fix build against recent syncthing.

* Sat Jun 17 2017 Fabio Valentini <decathorpe@gmail.com> - 0.8.7-1
- Update to version 0.8.7.

* Fri Jun 16 2017 Fabio Valentini <decathorpe@gmail.com> - 0.8.6-1
- Update to version 0.8.6.

* Tue Mar 07 2017 Fabio Valentini <decathorpe@gmail.com> - 0.8.5-4
- Clean up spec file.

* Fri Feb 10 2017 Fabio Valentini <decathorpe@gmail.com> - 0.8.5-3
- Do version tag in a future-proof way.

* Fri Feb 10 2017 Fabio Valentini <decathorpe@gmail.com> - 0.8.5-2
- Tag build with version info.

* Thu Feb 09 2017 Fabio Valentini <decathorpe@gmail.com> - 0.8.5-1
- Update to version 0.8.5.

* Sun Jan 22 2017 Fabio Valentini <decathorpe@gmail.com> - 0.8.4-1
- Update to version 0.8.4.

* Wed Jan 11 2017 Fabio Valentini <decathorpe@gmail.com> - 0.8.3-2
- Supplement syncthing.

* Mon Jul 18 2016 Fabio Valentini <decathorpe@gmail.com> - 0.8.3-1
- Update to version 0.8.3.

* Fri Jun 17 2016 Fabio Valentini <decathorpe@gmail.com> - 0.8.2-1
- Add BR: syncthing-devel now that I made it available.

* Sun Jun 12 2016 Fabio Valentini <decathorpe@gmail.com> - 0.8.2-1
- Update to version 0.8.2.

* Tue May 17 2016 Fabio Valentini <decathorpe@gmail.com> - 0.8-1
- Update to version 0.8.

* Sun Apr 10 2016 Fabio Valentini <decathorpe@gmail.com> - 0.7-2
- Fix version usage in scripts.

* Sun Apr 10 2016 Fabio Valentini <decathorpe@gmail.com> - 0.7-1
- Update to version 0.7.

* Wed Feb 24 2016 Fabio Valentini <decathorpe@gmail.com> - 0.6.8-4
- undo fix rhel build. doesn't work.

* Wed Feb 24 2016 Fabio Valentini <decathorpe@gmail.com> - 0.6.8-3
- Try to fix rhel7 build.

* Tue Feb 23 2016 Fabio Valentini <decathorpe@gmail.com> - 0.6.8-2
- rebuild for golang1.6

* Mon Jan 18 2016 Fabio Valentini <decathorpe@gmail.com> - 0.6.8-1
- First package for fedora.

